#! /usr/bin/env python
# encoding: utf-8

import os,sys
import Configure,Options,Utils
import ccroot,ar
from Configure import conftest
def find_gcc(conf):
	v=conf.env
	cc=None
	if v['CC']:cc=v['CC']
	elif'CC'in os.environ:cc=os.environ['CC']
	if not cc:cc=conf.find_program('gcc',var='CC')
	if not cc:cc=conf.find_program('cc',var='CC')
	if not cc:conf.fatal('gcc was not found')
	try:
		if Utils.cmd_output('%s --version'%cc).find('gcc')<0:
			conf.fatal('gcc was not found, see the result of gcc --version')
	except ValueError:
		conf.fatal('gcc --version could not be executed')
	v['CC']=cc
	v['CC_NAME']='gcc'
	ccroot.get_cc_version(conf,cc,'CC_VERSION')
def gcc_common_flags(conf):
	v=conf.env
	v['CC_SRC_F']=''
	v['CC_TGT_F']='-c -o '
	v['CPPPATH_ST']='-I%s'
	if not v['LINK_CC']:v['LINK_CC']=v['CC']
	v['CCLNK_SRC_F']=''
	v['CCLNK_TGT_F']='-o '
	v['LIB_ST']='-l%s'
	v['LIBPATH_ST']='-L%s'
	v['STATICLIB_ST']='-l%s'
	v['STATICLIBPATH_ST']='-L%s'
	v['RPATH_ST']='-Wl,-rpath,%s'
	v['CCDEFINES_ST']='-D%s'
	v['SHLIB_MARKER']='-Wl,-Bdynamic'
	v['STATICLIB_MARKER']='-Wl,-Bstatic'
	v['program_PATTERN']='%s'
	v['shlib_CCFLAGS']=['-fPIC','-DPIC']
	v['shlib_LINKFLAGS']=['-shared']
	v['shlib_PATTERN']='lib%s.so'
	v['staticlib_LINKFLAGS']=['-Wl,-Bstatic']
	v['staticlib_PATTERN']='lib%s.a'
	v['MACBUNDLE_LINKFLAGS']=['-bundle','-undefined dynamic_lookup']
	v['MACBUNDLE_CCFLAGS']=['-fPIC']
	v['MACBUNDLE_PATTERN']='%s.bundle'
def gcc_modifier_win32(conf):
	v=conf.env
	if sys.platform!='win32':return
	v['program_PATTERN']='%s.exe'
	v['shlib_PATTERN']='lib%s.dll'
	v['shlib_CCFLAGS']=[]
	v['staticlib_LINKFLAGS']=[]
def gcc_modifier_cygwin(conf):
	v=conf.env
	if sys.platform!='cygwin':return
	v['program_PATTERN']='%s.exe'
	v['shlib_PATTERN']='lib%s.dll'
	v['shlib_CCFLAGS']=[]
	v['staticlib_LINKFLAGS']=[]
def gcc_modifier_darwin(conf):
	v=conf.env
	if sys.platform!='darwin':return
	v['shlib_CCFLAGS']=['-fPIC','-compatibility_version 1','-current_version 1']
	v['shlib_LINKFLAGS']=['-dynamiclib']
	v['shlib_PATTERN']='lib%s.dylib'
	v['staticlib_LINKFLAGS']=[]
	v['SHLIB_MARKER']=''
	v['STATICLIB_MARKER']=''
def gcc_modifier_aix5(conf):
	v=conf.env
	if sys.platform!='aix5':return
	v['program_LINKFLAGS']=['-Wl,-brtl']
	v['shlib_LINKFLAGS']=['-shared','-Wl,-brtl,-bexpfull']
	v['SHLIB_MARKER']=''
detect='''
find_gcc
find_cpp
find_ar
gcc_common_flags
gcc_modifier_win32
gcc_modifier_cygwin
gcc_modifier_darwin
gcc_modifier_aix5
cc_load_tools
cc_add_flags
'''

conftest(find_gcc)
conftest(gcc_common_flags)
conftest(gcc_modifier_win32)
conftest(gcc_modifier_cygwin)
conftest(gcc_modifier_darwin)
conftest(gcc_modifier_aix5)



#! /usr/bin/env python
# encoding: utf-8

import os,types,imp,cPickle
import Environment,Utils,Options
from Logs import warn
from Constants import*
class ConfigurationError(Utils.WscriptError):
	pass
autoconfig=False
def find_file(filename,path_list):
	if type(path_list)is types.StringType:
		lst=path_list.split()
	else:
		lst=path_list
	for directory in lst:
		if os.path.exists(os.path.join(directory,filename)):
			return directory
	return''
def find_program_impl(env,filename,path_list=[],var=None):
	try:path_list=path_list.split()
	except AttributeError:pass
	if var:
		if var in os.environ:env[var]=os.environ[var]
		if env[var]:return env[var]
	if not path_list:path_list=os.environ['PATH'].split(os.pathsep)
	ext=(Options.platform=='win32')and'.exe,.com,.bat,.cmd'or''
	for y in[filename+x for x in ext.split(',')]:
		for directory in path_list:
			x=os.path.join(directory,y)
			if os.path.isfile(x):
				x=Utils.quote_whitespace(x)
				if var:env[var]=x
				return x
	return''
class ConfigurationContext(object):
	tests={}
	error_handlers=[]
	def __init__(self,env=None,blddir='',srcdir=''):
		self.env=None
		self.envname=''
		self.line_just=40
		self.blddir=blddir
		self.srcdir=srcdir
		self.cachedir=os.path.join(blddir,CACHE_DIR)
		self.all_envs={}
		self.defines={}
		self.cwd=os.getcwd()
		self.tools=[]
		self.setenv(DEFAULT)
		self.lastprog=''
		self.hash=0
		self.files=[]
		path=os.path.join(self.blddir,WAF_CONFIG_LOG)
		try:os.unlink(path)
		except(OSError,IOError):pass
		self.log=open(path,'wb')
	def __del__(self):
		if hasattr(self,'log')and self.log:
			self.log.close()
	def fatal(self,msg):
		raise ConfigurationError(msg)
	def check_tool(self,input,tooldir=None,funs=None):
		tools=Utils.to_list(input)
		if tooldir:tooldir=Utils.to_list(tooldir)
		for tool in tools:
			try:
				file,name,desc=imp.find_module(tool,tooldir)
			except ImportError:
				self.fatal("no tool named '%s' found."%tool)
			module=imp.load_module(tool,file,name,desc)
			func=getattr(module,'detect',None)
			if func:
				if type(func)is types.FunctionType:func(self)
				else:self.eval_rules(funs or func)
			self.tools.append({'tool':tool,'tooldir':tooldir,'funs':funs})
	def sub_config(self,dir):
		current=self.cwd
		self.cwd=os.path.join(self.cwd,dir)
		cur=os.path.join(self.cwd,WSCRIPT_FILE)
		mod=Utils.load_module(cur)
		if not hasattr(mod,'configure'):
			self.fatal('the module %s has no configure function; make sure such a function is defined'%cur)
		ret=mod.configure(self)
		global autoconfig
		if autoconfig:
			self.hash=Utils.hash_function_with_globals(self.hash,mod.configure)
			self.files.append(os.path.abspath(cur))
		self.cwd=current
		return ret
	def store(self,file=''):
		if not os.path.isdir(self.cachedir):
			os.makedirs(self.cachedir)
		if not file:
			file=open(os.path.join(self.cachedir,'build.config.py'),'w')
		file.write('version = 0x%x\n'%HEXVERSION)
		file.write('tools = %r\n'%self.tools)
		file.close()
		if not self.all_envs:
			self.fatal('nothing to store in the configuration context!')
		for key in self.all_envs:
			tmpenv=self.all_envs[key]
			tmpenv.store(os.path.join(self.cachedir,key+CACHE_SUFFIX))
	def set_env_name(self,name,env):
		self.all_envs[name]=env
		return env
	def retrieve(self,name,fromenv=None):
		try:
			env=self.all_envs[name]
		except KeyError:
			env=Environment.Environment()
			self.all_envs[name]=env
		else:
			if fromenv:warn("The environment %s may have been configured already"%name)
		return env
	def setenv(self,name):
		self.env=self.retrieve(name)
		self.envname=name
	def add_os_flags(self,var,dest=None):
		if not dest:dest=var
		try:self.env[dest]=os.environ[var]
		except KeyError:pass
	def check_message_1(self,sr):
		self.line_just=max(self.line_just,len(sr))
		self.log.write(sr+'\n\n')
		print"%s :"%sr.ljust(self.line_just),
	def check_message_2(self,sr,color='GREEN'):
		Utils.pprint(color,sr)
	def check_message(self,th,msg,state,option=''):
		sr='Checking for %s %s'%(th,msg)
		self.check_message_1(sr)
		p=self.check_message_2
		if state:p('ok '+option)
		else:p('not found','YELLOW')
	def check_message_custom(self,th,msg,custom,option='',color='PINK'):
		sr='Checking for %s %s'%(th,msg)
		self.check_message_1(sr)
		self.check_message_2(custom,color)
	def find_program(self,filename,path_list=[],var=None):
		ret=find_program_impl(self.env,filename,path_list,var)
		self.check_message('program',filename,ret,ret)
		return ret
	def __getattr__(self,name):
		r=self.__class__.__dict__.get(name,None)
		if r:return r
		if name and name.startswith('require_'):
			for k in['check_','find_']:
				n=name.replace('require_',k)
				ret=self.__class__.__dict__.get(n,None)
				if ret:
					def run(*k,**kw):
						r=ret(self,*k,**kw)
						if not r:
							self.fatal('requirement failure')
						return r
					return run
		self.fatal('No such method %r'%name)
	def eval_rules(self,rules):
		self.rules=Utils.to_list(rules)
		for x in self.rules:
			f=getattr(self,x)
			try:
				f()
			except Exception,e:
				ret=self.err_handler(x,e)
				if ret==BREAK:
					break
				elif ret==CONTINUE:
					continue
				else:
					raise
	def err_handler(self,fun,error):
		pass
def conf(f):
	setattr(ConfigurationContext,f.__name__,f)
	return f
def conftest(f):
	ConfigurationContext.tests[f.__name__]=f
	return conf(f)




#! /usr/bin/env python
# encoding: utf-8

import os,sys,re
import TaskGen,Utils,preproc,Logs,Build,Options
from Logs import error,debug,warn
from Utils import md5
from TaskGen import taskgen,after,before,feature
from Constants import*
import config_c
get_version_re=re.compile('\d+\.\d+(\.?\d+)*')
def get_cc_version(conf,cc,version_var):
	v=conf.env
	output=Utils.cmd_output('%s -dumpversion'%cc)
	if output:
		match=get_version_re.search(output)
		if match:
			v[version_var]=match.group(0)
			conf.check_message('compiler','version',1,v[version_var])
			return v[version_var]
	warn('could not determine the compiler version')
class DEBUG_LEVELS:
	ULTRADEBUG="ultradebug"
	DEBUG="debug"
	RELEASE="release"
	OPTIMIZED="optimized"
	CUSTOM="custom"
	ALL=[ULTRADEBUG,DEBUG,RELEASE,OPTIMIZED,CUSTOM]
def scan(self):
	debug('ccroot: _scan_preprocessor(self, node, env, path_lst)')
	all_nodes=[]
	all_names=[]
	seen=[]
	for node in self.inputs:
		gruik=preproc.c_parser(nodepaths=self.env['INC_PATHS'],defines=self.defines)
		gruik.start(node,self.env)
		if Logs.verbose:
			debug('deps: nodes found for %s: %s %s'%(str(node),str(gruik.nodes),str(gruik.names)))
			debug('deps: deps found for %s: %s'%(str(node),str(gruik.deps)))
		for x in gruik.nodes:
			if id(x)in seen:continue
			seen.append(id(x))
			all_nodes.append(x)
		for x in gruik.names:
			if not x in all_names:
				all_names.append(x)
	return(all_nodes,gruik.names)
class ccroot_abstract(TaskGen.task_gen):
	def __init__(self,*k,**kw):
		TaskGen.task_gen.__init__(self,*k,**kw)
		if len(k)>1:
			self.features.append('c'+k[1])
def default_cc(self):
	Utils.def_attrs(self,includes='',defines='',rpaths='',uselib='',uselib_local='',add_objects='',p_flag_vars=[],p_type_vars=[],scanner_defines={},compiled_tasks=[],link_task=None)
def get_target_name(self):
	tp='program'
	for x in self.features:
		if x in['cshlib','cstaticlib']:
			tp=x.lstrip('c')
	pattern=self.env[tp+'_PATTERN']
	if not pattern:pattern='%s'
	name=self.target
	k=name.rfind('/')
	return name[0:k+1]+pattern%name[k+1:]
def apply_verif(self):
	if not'objects'in self.features:
		if not self.source:
			raise Utils.WafError('no source files specified for %s'%self)
		if not self.target:
			raise Utils.WafError('no target for %s'%self)
def install_shlib(self):
	nums=self.vnum.split('.')
	path=self.install_path
	if not path:return
	libname=self.outputs[0].name
	name3=libname+'.'+self.vnum
	name2=libname+'.'+nums[0]
	name1=libname
	filename=self.outputs[0].abspath(self.env)
	bld=Build.bld
	bld.install_as(os.path.join(path,name3),filename,env=self.env)
	bld.symlink_as(os.path.join(path,name2),name3)
	bld.symlink_as(os.path.join(path,name1),name3)
def vars_target_cprogram(self):
	self.default_install_path='${PREFIX}/bin'
	self.default_chmod=0755
def vars_target_cstaticlib(self):
	self.default_install_path='${PREFIX}/lib'
def install_target_cstaticlib(self):
	if not Options.is_install:return
	self.link_task.install_path=self.install_path
def install_target_cshlib(self):
	if getattr(self,'vnum','')and sys.platform!='win32':
		tsk=self.link_task
		tsk.vnum=self.vnum
		tsk.install=install_shlib
def apply_incpaths(self):
	lst=[]
	for lib in self.to_list(self.uselib):
		for path in self.env['CPPPATH_'+lib]:
			if not path in lst:
				lst.append(path)
	if preproc.go_absolute:
		for path in preproc.standard_includes:
			if not path in lst:
				lst.append(path)
	for path in self.to_list(self.includes):
		if not path in lst:
			self.env.prepend_value('CPPPATH',path)
			if preproc.go_absolute or not os.path.isabs(path):
				lst.append(path)
	tree=Build.bld
	inc_lst=[]
	for path in lst:
		node=None
		if os.path.isabs(path):
			if preproc.go_absolute:
				node=Build.bld.root.find_dir(path)
		else:
			node=self.path.find_dir(path)
		if node:
			inc_lst.append(node)
	self.env['INC_PATHS']+=inc_lst
def apply_type_vars(self):
	for x in self.features:
		if not x in['cprogram','cstaticlib','cshlib']:
			continue
		x=x.lstrip('c')
		st=self.env[x+'_USELIB']
		if st:self.uselib=self.uselib+' '+st
		for var in self.p_type_vars:
			compvar='%s_%s'%(x,var)
			value=self.env[compvar]
			if value:self.env.append_value(var,value)
def apply_link(self):
	link=getattr(self,'link',None)
	if not link:
		if'cstaticlib'in self.features:link='ar_link_static'
		elif'cxx'in self.features:link='cxx_link'
		else:link='cc_link'
		if getattr(self,'vnum',None):link='vnum_'+link
	linktask=self.create_task(link)
	outputs=[t.outputs[0]for t in self.compiled_tasks]
	linktask.set_inputs(outputs)
	linktask.set_outputs(self.path.find_or_declare(get_target_name(self)))
	linktask.chmod=self.chmod
	self.link_task=linktask
def apply_lib_vars(self):
	env=self.env
	uselib=self.to_list(self.uselib)
	seen=[]
	names=self.to_list(self.uselib_local)[:]
	while names:
		x=names.pop(0)
		if x in seen:
			continue
		y=self.name_to_obj(x)
		if not y:
			raise Utils.WafError("object '%s' was not found in uselib_local (required by '%s')"%(x,self.name))
		if getattr(y,'uselib_local',None):
			lst=y.to_list(y.uselib_local)
			for u in lst:
				if not u in seen:
					names.append(u)
		y.post()
		seen.append(x)
		if'cshlib'in y.features or'cprogram'in y.features:
			env.append_value('LIB',y.target)
		elif'cstaticlib'in y.features:
			env.append_value('STATICLIB',y.target)
		tmp_path=y.path.bldpath(self.env)
		if not tmp_path in env['LIBPATH']:env.prepend_value('LIBPATH',tmp_path)
		if y.link_task is not None:
			self.link_task.set_run_after(y.link_task)
			dep_nodes=getattr(self.link_task,'dep_nodes',[])
			self.link_task.dep_nodes=dep_nodes+y.link_task.outputs
		morelibs=y.to_list(y.uselib)
		for v in morelibs:
			if v in uselib:continue
			uselib=[v]+uselib
		if getattr(y,'export_incdirs',None):
			cpppath_st=self.env['CPPPATH_ST']
			for x in self.to_list(y.export_incdirs):
				node=y.path.find_dir(x)
				if not node:raise Utils.WafError('object %s: invalid folder %s in export_incdirs'%(y.target,x))
				self.env.append_unique('INC_PATHS',node)
	for x in uselib:
		for v in self.p_flag_vars:
			val=self.env[v+'_'+x]
			if val:self.env.append_value(v,val)
def apply_objdeps(self):
	seen=[]
	names=self.to_list(self.add_objects)
	while names:
		x=names[0]
		if x in seen:
			names=names[1:]
			continue
		y=self.name_to_obj(x)
		if not y:
			raise Utils.WafError("object '%s' was not found in uselib_local (required by add_objects '%s')"%(x,self.name))
		if y.add_objects:
			added=0
			lst=y.to_list(y.add_objects)
			lst.reverse()
			for u in lst:
				if u in seen:continue
				added=1
				names=[u]+names
			if added:continue
		y.post()
		seen.append(x)
		self.link_task.inputs+=y.out_nodes
def apply_obj_vars(self):
	v=self.env
	lib_st=v['LIB_ST']
	staticlib_st=v['STATICLIB_ST']
	libpath_st=v['LIBPATH_ST']
	staticlibpath_st=v['STATICLIBPATH_ST']
	rpath_st=v['RPATH_ST']
	app=v.append_unique
	if v['FULLSTATIC']:
		v.append_value('LINKFLAGS',v['FULLSTATIC_MARKER'])
	for i in v['RPATH']:
		if i and rpath_st:
			app('LINKFLAGS',rpath_st%i)
	for i in v['LIBPATH']:
		app('LINKFLAGS',libpath_st%i)
		app('LINKFLAGS',staticlibpath_st%i)
	if v['STATICLIB']:
		v.append_value('LINKFLAGS',v['STATICLIB_MARKER'])
		k=[(staticlib_st%i)for i in v['STATICLIB']]
		app('LINKFLAGS',k)
	if not v['FULLSTATIC']:
		if v['STATICLIB']or v['LIB']:
			v.append_value('LINKFLAGS',v['SHLIB_MARKER'])
	app('LINKFLAGS',[lib_st%i for i in v['LIB']])
def apply_vnum(self):
	if sys.platform!='darwin'and sys.platform!='win32':
		try:
			nums=self.vnum.split('.')
		except AttributeError:
			pass
		else:
			try:name3=self.soname
			except AttributeError:name3=self.link_task.outputs[0].name+'.'+nums[0]
			self.link_task.outputs.append(self.link_task.outputs[0].parent.find_or_declare(name3))
			self.env.append_value('LINKFLAGS','-Wl,-h,'+name3)
def process_obj_files(self):
	if not hasattr(self,'obj_files'):return
	for x in self.obj_files:
		node=self.path.find_resource(x)
		self.link_task.inputs.append(node)
def add_obj_file(self,file):
	if not hasattr(self,'obj_files'):self.obj_files=[]
	if not'process_obj_files'in self.meths:self.meths.append('process_obj_files')
	self.obj_files.append(file)
def make_objects_available(self):
	self.out_nodes=[]
	app=self.out_nodes.append
	for t in self.compiled_tasks:app(t.outputs[0])
c_attrs={'cxxflag':'CXXFLAGS','cflag':'CCFLAGS','ccflag':'CCFLAGS','linkflag':'LINKFLAGS','ldflag':'LINKFLAGS','lib':'LIB','libpath':'LIBPATH','staticlib':'STATICLIB','staticlibpath':'STATICLIBPATH','rpath':'RPATH',}
def add_extra_flags(self):
	for x in self.__dict__.keys():
		y=x.lower()
		if y[-1]=='s':
			y=y[:-1]
		if c_attrs.get(y,None):
			self.env.append_unique(c_attrs[y],getattr(self,x))

feature('cc','cxx')(default_cc)
before('init_cc','init_cxx')(default_cc)
feature('cprogram','dprogram','cstaticlib','dstaticlib','cshlib','dshlib')(apply_verif)
feature('cprogram','dprogram')(vars_target_cprogram)
before('apply_core')(vars_target_cprogram)
feature('cstaticlib','dstaticlib','cshlib','dshlib')(vars_target_cstaticlib)
before('apply_core')(vars_target_cstaticlib)
feature('cprogram','dprogram','cstaticlib','dstaticlib','cshlib','dshlib')(install_target_cstaticlib)
after('apply_objdeps','apply_link')(install_target_cstaticlib)
feature('cshlib','dshlib')(install_target_cshlib)
after('apply_objdeps','apply_link')(install_target_cshlib)
feature('cc','cxx')(apply_incpaths)
after('apply_type_vars')(apply_incpaths)
feature('cc','cxx')(apply_type_vars)
taskgen(apply_link)
feature('cprogram','cshlib','cstaticlib')(apply_link)
after('apply_core')(apply_link)
feature('cc','cxx')(apply_lib_vars)
after('apply_vnum')(apply_lib_vars)
feature('objects')(apply_objdeps)
after('apply_obj_vars','apply_vnum')(apply_objdeps)
feature('cprogram','cshlib','cstaticlib')(apply_obj_vars)
after('apply_lib_vars')(apply_obj_vars)
feature('cprogram','cshlib','cstaticlib')(apply_vnum)
after('apply_link')(apply_vnum)
taskgen(process_obj_files)
after('apply_link')(process_obj_files)
taskgen(add_obj_file)
feature('objects')(make_objects_available)
after('apply_core')(make_objects_available)
taskgen(add_extra_flags)
feature('cc','cxx')(add_extra_flags)
before('init_cxx','init_cc')(add_extra_flags)



# This file is part of the Hotwire Shell user interface.
#   
# Copyright (C) 2007 Colin Walters <walters@verbum.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os,sys,threading,pty,logging

import gtk, gobject, pango
import vte

try:
    import gconf
    gconf_available = True
except:
    gconf_available = False

_logger = logging.getLogger("hotssh.VteTerminal")

class VteTerminalScreen(gtk.Bin):
    def __init__(self):
        super(VteTerminalScreen, self).__init__()
        self.term = vte.Terminal()
        self.__termbox = gtk.HBox()
        self.__scroll = gtk.VScrollbar(self.term.get_adjustment())
        border = gtk.Frame()
        border.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        border.add(self.term)
        self.__termbox.pack_start(border)
        self.__termbox.pack_start(self.__scroll, False)
        self.add(self.__termbox)

    def do_size_request(self, req):
        (w,h) = self.__termbox.size_request()
        req.width = w
        req.height = h

    def do_size_allocate(self, alloc):
        self.allocation = alloc
        wid_req = self.__termbox.size_allocate(alloc)

gobject.type_register(VteTerminalScreen)

# From gnome-terminal src/terminal-screen.c
_USERCHARS = "-A-Za-z0-9"
_PASSCHARS = "-A-Za-z0-9,?;.:/!%$^*&~\"#'"
_HOSTCHARS = "-A-Za-z0-9"
_PATHCHARS = "-A-Za-z0-9_$.+!*(),;:@&=?/~#%"
_SCHEME = "(news:|telnet:|nntp:|file:/|https?:|ftps?:|webcal:)"
_USER = "[" + _USERCHARS + "]+(:[" + _PASSCHARS + "]+)?"
_URLPATH = "/[" + _PATHCHARS + "]*[^]'.}>) \t\r\n,\\\"]"

class VteTerminalWidget(gtk.VBox):
    __gsignals__ = {
        "child-exited" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        "fork-child" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),        
    }    
    def __init__(self, cwd=None, cmd=None, ptyfd=None, initbuf=None, **kwargs):
        super(VteTerminalWidget, self).__init__()
        
        self.__screen = screen = VteTerminalScreen()
        self.__term = screen.term
        self.pack_start(screen, expand=True)
        
        self.pid = None
        self.exited = False
        
        self.__actions = [
            ('Copy', 'gtk-copy', _('_Copy'), '<control><shift>C', _('Copy selected text'), self.__copy_cb),
            ('Paste', 'gtk-paste', _('_Paste'), '<control><shift>V', _('Paste text'), self.__paste_cb),
        ]
        self.__action_group = gtk.ActionGroup('TerminalActions')
        self.__action_group.add_actions(self.__actions)
        self.__copyaction = self.__action_group.get_action('Copy')
        self.__pasteaction = self.__action_group.get_action('Paste')         

        # Various defaults
        self.__term.set_emulation('xterm')
        self.__term.set_allow_bold(True)
        self.__term.set_size(80, 24)
        self.__term.set_scrollback_lines(1500)
        self.__term.set_mouse_autohide(True)
        
        self.__colors_default = True
        self.__term.set_default_colors()
        if gconf_available:
            self.__set_gnome_colors()
        else:
            self.__set_gtk_colors()        

        # Use Gnome font if available
        if gconf_available:
            gconf_client = gconf.client_get_default()
            def on_font_change(*args):
                mono_font = gconf_client.get_string('/desktop/gnome/interface/monospace_font_name')
                _logger.debug("Using font '%s'", mono_font)
                font_desc = pango.FontDescription(mono_font)
                self.__term.set_font(font_desc)                
            gconf_client.notify_add('/desktop/gnome/interface/monospace_font_name', on_font_change)
            on_font_change()
            
        self.__match_asis = self.__term.match_add("\\<" + _SCHEME + "//(" + _USER + "@)?[" + _HOSTCHARS + ".]+" + \
                                                  "(:[0-9]+)?(" + _URLPATH + ")?\\>/?")

        self.__match_http = self.__term.match_add("\\<(www|ftp)[" + _HOSTCHARS + "]*\\.["  + _HOSTCHARS + ".]+" + \
                                                  "(:[0-9]+)?(" + _URLPATH + ")?\\>/?")
        
        self.__term.connect('button-press-event', self.__on_button_press)
        self.__term.connect('selection-changed', self.__on_selection_changed)
        self.__on_selection_changed()

        if ptyfd is not None:
            # If we have a PTY, set it up immediately
            self.__idle_do_cmd_fork(None, cwd, ptyfd, initbuf)
        else:
            # http://code.google.com/p/hotwire-shell/issues/detail?id=35
            # We do the command in an idle to hopefully have more state set up by then;
            # For example, "top" seems to be sized correctly on the first display
            # this way            
            gobject.timeout_add(250, self.__idle_do_cmd_fork, cmd, cwd, ptyfd, initbuf)
            
    def __idle_do_cmd_fork(self, cmd, cwd, ptyfd, initbuf):
        _logger.debug("Forking cmd: %r", cmd)
        self.__term.connect("child-exited", self._on_child_exited)
        if cwd:
            kwargs = {'directory': cwd}
        else:
            kwargs = {}
        if ptyfd:
            self.__term.set_pty(ptyfd)
            pid = None
        elif cmd:
            pid = self.__term.fork_command(cmd[0], cmd, **kwargs)
        else:
            pid = self.__term.fork_command(**kwargs)
        if initbuf is not None:
            self.__term.feed(initbuf)            
        self.pid = pid
        self.emit('fork-child')
        
    def __on_button_press(self, term, event):
        match = self.__term.match_check(int(event.x/term.get_char_width()), int(event.y/term.get_char_height()))
        if event.button == 1 and event.state & gtk.gdk.CONTROL_MASK:
            if not match:
                return
            (matchstr, mdata) = match            
            if mdata == self.__match_http:
                url = 'http://' + matchstr
            else:
                url = matchstr
            self.__open_url(url)
            return True
        elif event.button == 3:
            menu = gtk.Menu()
            menuitem = self.__copyaction.create_menu_item()
            menu.append(menuitem)
            menuitem = self.__pasteaction.create_menu_item()
            menu.append(menuitem)
            if match:
                (matchstr, mdata) = match
                menuitem = gtk.ImageMenuItem(_('Open Link'))
                menuitem.set_property('image', gtk.image_new_from_stock('gtk-go-to', gtk.ICON_SIZE_MENU))
                menuitem.connect('activate', lambda menu: self.__open_url(url))
                menu.append(gtk.SeparatorMenuItem())
                menu.append(menuitem)
            extra = self._get_extra_context_menuitems()
            if len(extra) > 0:
                menu.append(gtk.SeparatorMenuItem())
                for item in extra:
                    menu.append(item)
            menu.show_all()
            menu.popup(None, None, None, event.button, event.time)            
            return True
        return False

    def _get_extra_context_menuitems(self):
        return []

    def __open_url(self, url):
        # Older webbrowser.py didn't check gconf
        from hotwire.sysdep import is_windows
        if sys.version_info[0] == 2 and sys.version_info[1] < 6 and (not is_windows()):
            try:
                import hotwire.externals.webbrowser as webbrowser
            except ImportError, e:
                _logger.warn("Couldn't import hotwire.externals.webbrowser", exc_info=True)
                import webbrowser
        else:
            import webbrowser            
        webbrowser.open(url)        
            
    def __on_selection_changed(self, *args):
        have_selection = self.__term.get_has_selection()
        self.__copyaction.set_sensitive(have_selection)

    def __copy_cb(self, a):
        _logger.debug("doing copy")
        self.__term.copy_clipboard()

    def __paste_cb(self, a):
        _logger.debug("doing paste")        
        self.__term.paste_clipboard()            
            
    def _on_child_exited(self, term):
        _logger.debug("Caught child exited")
        self.exited = True
        self.emit('child-exited')        
   
    def get_vte(self):
        return self.__term
    
    def get_action_group(self):
        return self.__action_group
    
    def set_copy_paste_actions(self, copyaction, pasteaction):
        """Useful in an environment where there is a global UI
        rather than a merged approach.""" 
        self.__copyaction = copyaction
        self.__pasteaction = pasteaction
        
    def __set_gnome_colors(self):
        gconf_client = gconf.client_get_default()        
        term_profile = '/apps/gnome-terminal/profiles/Default'
        fg_key = term_profile + '/foreground_color'
        bg_key = term_profile + '/background_color'
        def on_color_change(*args):
            if not self.__colors_default:
                return
            fg = gtk.gdk.color_parse(gconf_client.get_string(fg_key))
            self.set_color(True, fg, isdefault=True)
            bg = gtk.gdk.color_parse(gconf_client.get_string(bg_key))
            self.set_color(False, bg, isdefault=True)
        gconf_client.notify_add(fg_key, on_color_change)
        gconf_client.notify_add(bg_key, on_color_change)
        on_color_change()        
    
    def __set_gtk_colors(self):
        fg = self.style.text[gtk.STATE_NORMAL]
        bg = self.style.base[gtk.STATE_NORMAL]
        self.set_color(True, fg, isdefault=True)
        self.set_color(False, bg, isdefault=True)
        
    def set_color(self, is_foreground, color, isdefault=False):
        if not isdefault:
            self.__colors_default = False            
        if is_foreground:
            self.__term.set_color_foreground(color)
            self.__term.set_color_bold(color)
            self.__term.set_color_dim(color)            
        else:
            self.__term.set_color_background(color)




#! /usr/bin/env python
# encoding: utf-8

import os,sys,shutil,traceback,time
import Utils,Configure,Build,Logs,Options,Environment
from Logs import error,warn,info
from Constants import*
g_gz='bz2'
def add_subdir(dir,bld):
	try:bld.rescan(bld.path)
	except OSError:raise Utils.WscriptError("No such directory "+bld.path.abspath())
	old=bld.path
	new=bld.path.find_dir(dir)
	if new is None:
		raise Utils.WscriptError('subdir not found (%s), restore is %s'%(dir,bld.path))
	bld.path=new
	try:
		file_path=os.path.join(new.abspath(),WSCRIPT_BUILD_FILE)
		file=open(file_path,'r')
		exec file
		if file:file.close()
	except IOError:
		file_path=os.path.join(new.abspath(),WSCRIPT_FILE)
		module=Utils.load_module(file_path)
		module.build(bld)
	bld.path=old
def configure():
	err='The %s is not given in %s:\n * define a top level attribute named "%s"\n * run waf configure --%s=xxx'
	src=getattr(Options.options,SRCDIR,None)
	if not src:src=getattr(Utils.g_module,SRCDIR,None)
	if not src:raise Utils.WscriptError(err%(SRCDIR,os.path.abspath('.'),SRCDIR,SRCDIR))
	src=os.path.abspath(src)
	bld=getattr(Options.options,BLDDIR,None)
	if not bld:bld=getattr(Utils.g_module,BLDDIR,None)
	if not bld:raise Utils.WscriptError(err%(BLDDIR,os.path.abspath('.'),BLDDIR,BLDDIR))
	bld=os.path.abspath(bld)
	try:os.makedirs(bld)
	except OSError:pass
	conf=Configure.ConfigurationContext(srcdir=src,blddir=bld)
	conf.sub_config('')
	conf.store()
	env=Environment.Environment()
	env[BLDDIR]=bld
	env[SRCDIR]=src
	env['argv']=sys.argv
	env['hash']=conf.hash
	env['files']=conf.files
	env['commands']=Options.commands
	env['options']=Options.options.__dict__
	env.store(Options.lockfile)
def prepare_impl(t,cwd,ver,wafdir):
	Options.tooldir=[t]
	Options.launch_dir=cwd
	if'--version'in sys.argv:
		opt_obj=Options.Handler()
		opt_obj.parse_args()
		sys.exit(0)
	msg1='Waf: *** Nothing to do! Please run waf from a directory containing a file named "%s"'%WSCRIPT_FILE
	build_dir_override=None
	candidate=None
	cwd=Options.launch_dir
	lst=os.listdir(cwd)
	search_for_candidate=True
	if WSCRIPT_FILE in lst:
		candidate=cwd
	elif'configure'in sys.argv and not WSCRIPT_BUILD_FILE in lst:
		calldir=os.path.abspath(os.path.dirname(sys.argv[0]))
		if WSCRIPT_FILE in os.listdir(calldir):
			candidate=calldir
			search_for_candidate=False
		else:
			error('arg[0] directory does not contain a wscript file')
			sys.exit(1)
		build_dir_override=cwd
	while search_for_candidate:
		if len(cwd)<=3:
			break
		dirlst=os.listdir(cwd)
		if WSCRIPT_FILE in dirlst:
			candidate=cwd
		if'configure'in sys.argv and candidate:
			break
		if Options.lockfile in dirlst:
			break
		cwd=cwd[:cwd.rfind(os.sep)]
	if not candidate:
		if'-h'in sys.argv or'--help'in sys.argv:
			warn('No wscript file found: the help message may be incomplete')
			opt_obj=Options.Handler()
			opt_obj.parse_args()
		else:
			error(msg1)
		sys.exit(0)
	os.chdir(candidate)
	Utils.set_main_module(os.path.join(candidate,WSCRIPT_FILE))
	if build_dir_override:
		d=getattr(Utils.g_module,BLDDIR,None)
		if d:
			msg=' Overriding build directory %s with %s'%(d,build_dir_override)
			warn(msg)
		Utils.g_module.blddir=build_dir_override
	opt_obj=Options.Handler()
	opt_obj.sub_options('')
	opt_obj.parse_args()
	fun=getattr(Utils.g_module,'init',None)
	if fun:fun()
	for x in['dist','distclean','distcheck']:
		if Options.commands[x]:
			fun=getattr(Utils.g_module,x,None)
			if fun:
				fun()
			else:
				eval(x+'()')
			sys.exit(0)
	main()
def prepare(t,cwd,ver,wafdir):
	if WAFVERSION!=ver:
		msg='Version mismatch: waf %s <> wafadmin %s (wafdir %s)'%(ver,WAFVERSION,wafdir)
		print'\033[91mError: %s\033[0m'%msg
		sys.exit(1)
	try:
		prepare_impl(t,cwd,ver,wafdir)
	except Utils.WafError,e:
		error(e)
		sys.exit(1)
def main():
	if Options.commands['configure']:
		ini=time.time()
		configure()
		ela=''
		if not Options.options.progress_bar:ela=time.strftime(' (%H:%M:%S)',time.gmtime(time.time()-ini))
		info('Configuration finished successfully%s; project is now ready to build.'%ela)
		sys.exit(0)
	bld=Build.BuildContext()
	try:
		proj=Environment.Environment(Options.lockfile)
	except IOError:
		if Options.commands['clean']:
			raise Utils.WafError("Nothing to clean (project not configured)")
		else:
			if Configure.autoconfig:
				warn("Reconfiguring the project")
				configure()
				bld=Build.BuildContext()
				proj=Environment.Environment(Options.lockfile)
			else:
				raise Utils.WafError("Project not configured (run 'waf configure' first)")
	if Configure.autoconfig:
		if not Options.commands['clean']and not Options.commands['uninstall']:
			reconf=0
			h=0
			try:
				for file in proj['files']:
					mod=Utils.load_module(file)
					h=Utils.hash_function_with_globals(h,mod.configure)
				reconf=(h!=proj['hash'])
			except Exception,ex:
				warn("Reconfiguring the project (an exception occurred: %s)"%(str(ex),))
				reconf=1
			if reconf:
				warn("Reconfiguring the project (the configuration has changed)")
				back=(Options.commands,Options.options,Logs.zones,Logs.verbose)
				Options.commands=proj['commands']
				Options.options.__dict__=proj['options']
				configure()
				(Options.commands,Options.options,Logs.zones,Logs.verbose)=back
				bld=Build.BuildContext()
				proj=Environment.Environment(Options.lockfile)
	bld.load_dirs(proj[SRCDIR],proj[BLDDIR])
	bld.load_envs()
	f=getattr(Utils.g_module,'build',None)
	if f:
		f(bld)
	else:
		main_wscript=None
		for(file_path,module)in Utils.g_loaded_modules.items():
			if module.__name__=='wscript_main':
				main_wscript=file_path
				break
		raise Utils.WscriptError("Could not find the function 'def build(bld).'",main_wscript)
	pre_build=getattr(Utils.g_module,'pre_build',None)
	if pre_build:pre_build()
	if Options.commands['build']or Options.is_install:
		if Options.commands['uninstall']:
			import Task
			def runnable_status(self):
				return SKIP_ME
			setattr(Task.Task,'runnable_status',runnable_status)
		ini=time.time()
		bld.compile()
		if Options.options.progress_bar:print''
		if Options.is_install:
			bld.install()
		ela=''
		if not Options.options.progress_bar:
			ela=time.strftime(' (%H:%M:%S)',time.gmtime(time.time()-ini))
		if Options.commands['install']:msg='Compilation and installation finished successfully%s'%ela
		elif Options.commands['uninstall']:msg='Uninstallation finished successfully%s'%ela
		else:msg='Compilation finished successfully%s'%ela
		info(msg)
	if Options.commands['clean']:
		try:
			bld.clean()
			info('Cleaning finished successfully')
		finally:
			bld.save()
	fun=getattr(Utils.g_module,'shutdown',None)
	if fun:fun()
excludes='.bzr .bzrignore .git .gitignore .svn CVS .cvsignore .arch-ids {arch} SCCS BitKeeper .hg Makefile Makefile.in config.log'.split()
dist_exts='~ .rej .orig .pyc .pyo .bak .tar.bz2 tar.gz .zip .swp'.split()
def dont_dist(name,src,build_dir):
	global excludes,dist_exts
	if(name.startswith(',,')or name.startswith('++')or name.startswith('.waf-1.')or(src=='.'and name==Options.lockfile)or name in excludes or name==build_dir):
		return True
	for ext in dist_exts:
		if name.endswith(ext):
			return True
	return False
def copytree(src,dst,build_dir):
	names=os.listdir(src)
	os.makedirs(dst)
	for name in names:
		srcname=os.path.join(src,name)
		dstname=os.path.join(dst,name)
		if dont_dist(name,src,build_dir):
			continue
		if os.path.isdir(srcname):
			copytree(srcname,dstname,build_dir)
		else:
			shutil.copy2(srcname,dstname)
def distclean():
	lst=os.listdir('.')
	for f in lst:
		if f==Options.lockfile:
			try:
				proj=Environment.Environment(f)
				shutil.rmtree(proj[BLDDIR])
			except(OSError,IOError):
				pass
			try:
				os.remove(f)
			except(OSError,IOError):
				pass
		if f.startswith('.waf-'):
			shutil.rmtree(f,ignore_errors=True)
	info('distclean finished successfully')
def dist(appname='',version=''):
	import tarfile
	if not appname:appname=getattr(Utils.g_module,APPNAME,'noname')
	if not version:version=getattr(Utils.g_module,VERSION,'1.0')
	tmp_folder=appname+'-'+version
	arch_name=tmp_folder+'.tar.'+g_gz
	try:
		shutil.rmtree(tmp_folder)
	except(OSError,IOError):
		pass
	try:
		os.remove(arch_name)
	except(OSError,IOError):
		pass
	copytree('.',tmp_folder,getattr(Utils.g_module,BLDDIR,None))
	dist_hook=getattr(Utils.g_module,'dist_hook',None)
	if dist_hook:
		os.chdir(tmp_folder)
		try:
			dist_hook()
		finally:
			os.chdir('..')
	tar=tarfile.open(arch_name,'w:'+g_gz)
	tar.add(tmp_folder)
	tar.close()
	info('Your archive is ready -> %s'%arch_name)
	if os.path.exists(tmp_folder):shutil.rmtree(tmp_folder)
	return arch_name
def distcheck(appname='',version=''):
	import tempfile,tarfile
	import pproc
	if not appname:appname=getattr(Utils.g_module,APPNAME,'noname')
	if not version:version=getattr(Utils.g_module,VERSION,'1.0')
	waf=os.path.abspath(sys.argv[0])
	tarball=dist(appname,version)
	t=tarfile.open(tarball)
	for x in t:t.extract(x)
	t.close()
	instdir=tempfile.mkdtemp('.inst','%s-%s'%(appname,version))
	cwd_before=os.getcwd()
	retval=pproc.Popen('%(waf)s configure && %(waf)s'' && %(waf)s check && %(waf)s install --destdir=%(instdir)s'' && %(waf)s uninstall --destdir=%(instdir)s'%vars(),shell=True).wait()
	if retval:
		raise Utils.WafError('distcheck failed with code %i'%(retval))
	if os.path.exists(instdir):
		raise Utils.WafError('distcheck succeeded, but files were left in %s'%(instdir))
	else:
		info('distcheck finished successfully')




#! /usr/bin/env python
# encoding: utf-8

import TaskGen
from TaskGen import taskgen,feature
TaskGen.declare_chain(name='luac',action='${LUAC} -s -o ${TGT} ${SRC}',ext_in='.lua',ext_out='.luac',reentrant=0,install='LUADIR',)
def init_lua(self):
	self.default_chmod=0755
def detect(conf):
	luac=conf.find_program('luac',var='LUAC')
	if not luac:conf.fatal('cannot find the compiler "luac"')

taskgen(init_lua)
feature('lua')(init_lua)



#! /usr/bin/env python
# encoding: utf-8
import sys
if sys.hexversion < 0x020400f0: from sets import Set as set
import os,sys,imp,types,string,errno,traceback,inspect,re
from UserDict import UserDict
import Logs,pproc
from Constants import*
is_win32=sys.platform=='win32'
class WafError(Exception):
	def __init__(self,*args):
		self.args=args
		self.stack=traceback.extract_stack()
		Exception.__init__(self,*args)
	def __str__(self):
		return str(len(self.args)==1 and self.args[0]or self.args)
class WscriptError(WafError):
	def __init__(self,message,wscript_file=None):
		if wscript_file:
			self.wscript_file=wscript_file
			self.wscript_line=None
		else:
			(self.wscript_file,self.wscript_line)=self.locate_error()
		msg_file_line=''
		if self.wscript_file:
			msg_file_line="%s:"%self.wscript_file
			if self.wscript_line:
				msg_file_line+="%s:"%self.wscript_line
		err_message="%s error: %s"%(msg_file_line,message)
		WafError.__init__(self,err_message)
	def locate_error(self):
		stack=traceback.extract_stack()
		stack.reverse()
		for frame in stack:
			file_name=os.path.basename(frame[0])
			is_wscript=(file_name==WSCRIPT_FILE or file_name==WSCRIPT_BUILD_FILE)
			if is_wscript:
				return(frame[0],frame[1])
		return(None,None)
indicator=is_win32 and'\x1b[A\x1b[K%s%s%s\r'or'\x1b[K%s%s%s\r'
try:
	from fnv import new as md5
	import Constants
	Constants.SIG_NIL='signofnv'
	def h_file(filename):
		m=md5()
		try:
			m.hfile(filename)
			x=m.digest()
			if x is None:raise OSError,"not a file"
			return x
		except SystemError:
			raise OSError,"not a file"+filename
except ImportError:
	try:
		from hashlib import md5
	except ImportError:
		from md5 import md5
	def h_file(filename):
		f=file(filename,'rb')
		m=md5()
		readBytes=100000
		while(readBytes):
			readString=f.read(readBytes)
			m.update(readString)
			readBytes=len(readString)
		f.close()
		return m.digest()
class ordered_dict(UserDict):
	def __init__(self,dict=None):
		self.allkeys=[]
		UserDict.__init__(self,dict)
	def __delitem__(self,key):
		self.allkeys.remove(key)
		UserDict.__delitem__(self,key)
	def __setitem__(self,key,item):
		if key not in self.allkeys:self.allkeys.append(key)
		UserDict.__setitem__(self,key,item)
def exec_command(s,shell=1,log=None):
	proc=pproc.Popen(s,shell=shell,stdout=log,stderr=log)
	return proc.wait()
if is_win32:
	old_log=exec_command
	def exec_command(s,shell=1,log=None):
		if len(s)<2000:return old_log(s,shell=shell,log=log)
		startupinfo=pproc.STARTUPINFO()
		startupinfo.dwFlags|=pproc.STARTF_USESHOWWINDOW
		proc=pproc.Popen(s,shell=False,startupinfo=startupinfo)
		return proc.wait()
listdir=os.listdir
if is_win32:
	def listdir_win32(s):
		if re.match('^[A-Z]:$',s):
			s+=os.sep
		if not os.path.isdir(s):
			e=OSError()
			e.errno=errno.ENOENT
			raise e
		return os.listdir(s)
	listdir=listdir_win32
def waf_version(mini=0x010000,maxi=0x100000):
	ver=HEXVERSION
	try:min_val=mini+0
	except TypeError:min_val=int(mini.replace('.','0'),16)
	if min_val>ver:
		Logs.error("waf version should be at least %s (%s found)"%(mini,ver))
		sys.exit(0)
	try:max_val=maxi+0
	except TypeError:max_val=int(maxi.replace('.','0'),16)
	if max_val<ver:
		Logs.error("waf version should be at most %s (%s found)"%(maxi,ver))
		sys.exit(0)
def python_24_guard():
	if sys.hexversion<0x20400f0:
		raise ImportError,"Waf requires Python >= 2.3 but the raw source requires Python 2.4"
def ex_stack():
	exc_type,exc_value,tb=sys.exc_info()
	exc_lines=traceback.format_exception(exc_type,exc_value,tb)
	return''.join(exc_lines)
def to_list(sth):
	if type(sth)is types.ListType:
		return sth
	else:
		return sth.split()
g_loaded_modules={}
g_module=None
def load_module(file_path,name=WSCRIPT_FILE):
	try:
		return g_loaded_modules[file_path]
	except KeyError:
		pass
	module=imp.new_module(name)
	try:
		file=open(file_path,'r')
	except(IOError,OSError):
		raise WscriptError('The file %s could not be opened!'%file_path)
	module_dir=os.path.dirname(file_path)
	sys.path.insert(0,module_dir)
	exec file in module.__dict__
	sys.path.remove(module_dir)
	if file:file.close()
	g_loaded_modules[file_path]=module
	return module
def set_main_module(file_path):
	global g_module
	g_module=load_module(file_path,'wscript_main')
def to_hashtable(s):
	tbl={}
	lst=s.split('\n')
	for line in lst:
		if not line:continue
		mems=line.split('=')
		tbl[mems[0]]=mems[1]
	return tbl
def get_term_cols():
	return 80
try:
	import struct,fcntl,termios
except ImportError:
	pass
else:
	if sys.stdout.isatty():
		def myfun():
			dummy_lines,cols=struct.unpack("HHHH",fcntl.ioctl(sys.stdout.fileno(),termios.TIOCGWINSZ,struct.pack("HHHH",0,0,0,0)))[:2]
			return cols
		try:
			myfun()
		except IOError:
			pass
		else:
			get_term_cols=myfun
rot_idx=0
rot_chr=['\\','|','/','-']
def split_path(path):
	if not path:return['']
	return path.split('/')
if is_win32:
	def split_path(path):
		h,t=os.path.splitunc(path)
		if not h:return __split_dirs(t)
		return[h]+__split_dirs(t)[1:]
	def __split_dirs(path):
		h,t=os.path.split(path)
		if not h:return[t]
		if h==path:return[h.replace('\\','')]
		if not t:return __split_dirs(h)
		else:return __split_dirs(h)+[t]
def copy_attrs(orig,dest,names,only_if_set=False):
	for a in to_list(names):
		u=getattr(orig,a,())
		if u or not only_if_set:
			setattr(dest,a,u)
def def_attrs(self,**kw):
	for k,v in kw.iteritems():
		if not hasattr(self,k):
			setattr(self,k,v)
quote_define_name_table=None
def quote_define_name(path):
	global quote_define_name_table
	if not quote_define_name_table:
		invalid_chars=set([chr(x)for x in xrange(256)])-set(string.digits+string.uppercase)
		quote_define_name_table=string.maketrans(''.join(invalid_chars),'_'*len(invalid_chars))
	return string.translate(string.upper(path),quote_define_name_table)
def quote_whitespace(path):
	return(path.strip().find(' ')>0 and'"%s"'%path or path).replace('""','"')
def trimquotes(s):
	if not s:return''
	s=s.rstrip()
	if s[0]=="'"and s[-1]=="'":return s[1:-1]
	return s
def h_list(lst):
	m=md5()
	m.update(str(lst))
	return m.digest()
def h_fun(fun):
	try:
		return fun.code
	except AttributeError:
		try:
			h=inspect.getsource(fun)
		except IOError:
			h="nocode"
		try:
			fun.code=h
		except AttributeError:
			pass
		return h
_hash_blacklist_types=(types.BuiltinFunctionType,types.ModuleType,types.FunctionType,types.ClassType,types.TypeType,types.NoneType,)
def hash_function_with_globals(prevhash,func):
	assert type(func)is types.FunctionType
	for name,value in func.func_globals.iteritems():
		if type(value)in _hash_blacklist_types:
			continue
		if isinstance(value,type):
			continue
		try:
			prevhash=hash((prevhash,name,value))
		except TypeError:
			pass
	return hash((prevhash,inspect.getsource(func)))
def pprint(col,str,label=''):
	print"%s%s%s %s"%(Logs.colors(col),str,Logs.colors.NORMAL,label)
def check_dir(dir):
	try:
		os.stat(dir)
	except OSError:
		try:
			os.makedirs(dir)
		except OSError,e:
			raise WafError("Cannot create folder '%s' (original error: %s)"%(dir,e))
def cmd_output(cmd,e=None,silent=False):
	p=pproc.Popen(cmd,stdout=pproc.PIPE,shell=True,env=e)
	output=p.communicate()[0]
	if p.returncode:
		if not silent:
			msg="command execution failed: %s -> %r"%(cmd,str(output))
			raise ValueError,msg
		output=''
	return output
reg_subst=re.compile(r"(\\\\)|(\\\$)|\$\{([^}]+)\}")
def subst_vars(expr,params):
	def repl_var(m):
		if m.group(1):
			return'\\'
		if m.group(2):
			return'$'
		return params[m.group(3)]
	return reg_subst.sub(repl_var,expr)
def detect_platform():
	s=sys.platform
	for x in'cygwin linux irix sunos hpux aix darwin'.split():
		if s.find(x)>=0:
			return x
	if os.name in'posix java os2'.split():
		return os.name
	return s




#! /usr/bin/env python
# encoding: utf-8

import os,sys
import Task
from Configure import conftest
ar_str='${AR} ${ARFLAGS} ${TGT} ${SRC} && ${RANLIB} ${RANLIBFLAGS} ${TGT}'
if sys.platform=="win32":
	ar_str='${AR} s${ARFLAGS} ${TGT} ${SRC}'
cls=Task.simple_task_type('ar_link_static',ar_str,color='YELLOW',ext_in='.o')
cls.maxjobs=1
def detect(conf):
	comp=conf.find_program('ar',var='AR')
	if not comp:return
	ranlib=conf.find_program('ranlib',var='RANLIB')
	if not ranlib:return
	v=conf.env
	v['AR']=comp
	v['ARFLAGS']='rc'
	v['RANLIB']=ranlib
	v['RANLIBFLAGS']=''
def find_ar(conf):
	v=conf.env
	conf.check_tool('ar')
	if not v['AR']:conf.fatal('ar is required for static libraries - not found')
def find_cpp(conf):
	v=conf.env
	cpp=None
	if v['CPP']:cpp=v['CPP']
	elif'CPP'in os.environ:cpp=os.environ['CPP']
	if not cpp:cpp=conf.find_program('cpp',var='CPP')
	if not cpp:cpp=v['CC']
	if not cpp:cpp=v['CXX']
	v['CPP']=cpp
def cc_add_flags(conf):
	conf.add_os_flags('CFLAGS','CCFLAGS')
	conf.add_os_flags('CPPFLAGS')
	conf.add_os_flags('LINKFLAGS')
def cxx_add_flags(conf):
	conf.add_os_flags('CXXFLAGS')
	conf.add_os_flags('CPPFLAGS')
	conf.add_os_flags('LINKFLAGS')
def cc_load_tools(conf):
	conf.check_tool('cc')
def cxx_load_tools(conf):
	conf.check_tool('cxx')

conftest(find_ar)
conftest(find_cpp)
conftest(cc_add_flags)
conftest(cxx_add_flags)
conftest(cc_load_tools)
conftest(cxx_load_tools)



#! /usr/bin/env python
# encoding: utf-8

import re,sys,os,string,types
if __name__=='__main__':
	sys.path=['.','..']+sys.path
import Logs,Build,Utils
from Logs import debug,error
import traceback
class PreprocError(Utils.WafError):
	pass
POPFILE='-'
go_absolute=0
standard_includes=['/usr/include']
if sys.platform=="win32":
	standard_includes=[]
use_trigraphs=0
'apply the trigraph rules first'
strict_quotes=0
g_optrans={'not':'!','and':'&&','bitand':'&','and_eq':'&=','or':'||','bitor':'|','or_eq':'|=','xor':'^','xor_eq':'^=','compl':'~',}
re_lines=re.compile('^[ \t]*(#|%:)[ \t]*(ifdef|ifndef|if|else|elif|endif|include|import|define|undef|pragma)[ \t]*(.*)\r*$',re.IGNORECASE|re.MULTILINE)
re_mac=re.compile("^[a-zA-Z_]\w*")
re_fun=re.compile('^[a-zA-Z_][a-zA-Z0-9_]*[(]')
re_pragma_once=re.compile('^\s*once\s*',re.IGNORECASE)
re_nl=re.compile('\\\\\r*\n',re.MULTILINE)
re_cpp=re.compile(r"""(/\*[^*]*\*+([^/*][^*]*\*+)*/)|//[^\n]*|("(\\.|[^"\\])*"|'(\\.|[^'\\])*'|.[^/"'\\]*)""",re.MULTILINE)
trig_def=[('??'+a,b)for a,b in zip("=-/!'()<>",r'#~\|^[]{}')]
chr_esc={'0':0,'a':7,'b':8,'t':9,'n':10,'f':11,'v':12,'r':13,'\\':92,"'":39}
NUM='i'
OP='O'
IDENT='T'
STR='s'
CHAR='c'
tok_types=[NUM,STR,IDENT,OP]
exp_types=[r"""0[xX](?P<hex>[a-fA-F0-9]+)(?P<qual1>[uUlL]*)|L*?'(?P<char>(\\.|[^\\'])+)'|(?P<n1>\d+)[Ee](?P<exp0>[+-]*?\d+)(?P<float0>[fFlL]*)|(?P<n2>\d*\.\d+)([Ee](?P<exp1>[+-]*?\d+))?(?P<float1>[fFlL]*)|(?P<n4>\d+\.\d*)([Ee](?P<exp2>[+-]*?\d+))?(?P<float2>[fFlL]*)|(?P<oct>0*)(?P<n0>\d+)(?P<qual2>[uUlL]*)""",r'L?"([^"\\]|\\.)*"',r'[a-zA-Z_]\w*',r'%:%:|<<=|>>=|\.\.\.|<<|<%|<:|<=|>>|>=|\+\+|\+=|--|->|-=|\*=|/=|%:|%=|%>|==|&&|&=|\|\||\|=|\^=|:>|!=|##|[\(\)\{\}\[\]<>\?\|\^\*\+&=:!#;,%/\-\?\~\.]',]
re_clexer=re.compile('|'.join(["(?P<%s>%s)"%(name,part)for name,part in zip(tok_types,exp_types)]),re.M)
accepted='a'
ignored='i'
undefined='u'
skipped='s'
def repl(m):
	s=m.group(1)
	if s is not None:return' '
	s=m.group(3)
	if s is None:return''
	return s
def filter_comments(filename):
	f=open(filename,"r")
	code=f.read()
	f.close()
	if use_trigraphs:
		for(a,b)in trig_def:code=code.split(a).join(b)
	code=re_nl.sub('',code)
	code=re_cpp.sub(repl,code)
	return[(m.group(2),m.group(3))for m in re.finditer(re_lines,code)]
prec={}
ops=['* / %','+ -','<< >>','< <= >= >','== !=','& | ^','&& ||',',']
for x in range(len(ops)):
	syms=ops[x]
	for u in syms.split():
		prec[u]=x
def reduce_nums(val_1,val_2,val_op):
	try:a=0+val_1
	except TypeError:a=int(val_1)
	try:b=0+val_2
	except TypeError:b=int(val_2)
	d=val_op
	if d=='%':c=a%b
	elif d=='+':c=a+b
	elif d=='-':c=a-b
	elif d=='*':c=a*b
	elif d=='/':c=a/b
	elif d=='^':c=a^b
	elif d=='|':c=a|b
	elif d=='||':c=int(a or b)
	elif d=='&':c=a&b
	elif d=='&&':c=int(a and b)
	elif d=='==':c=int(a==b)
	elif d=='!=':c=int(a!=b)
	elif d=='<=':c=int(a<=b)
	elif d=='<':c=int(a<b)
	elif d=='>':c=int(a>b)
	elif d=='>=':c=int(a>=b)
	elif d=='^':c=int(a^b)
	elif d=='<<':c=a<<b
	elif d=='>>':c=a>>b
	else:c=0
	return c
def get_expr(lst,defs,ban):
	if not lst:return([],[],[])
	(p,v)=lst[0]
	if p==NUM:
		return(p,v,lst[1:])
	elif p==STR:
		try:
			(p2,v2)=lst[1]
			if p2==STR:return(p,v+v2,lst[2:])
		except IndexError:pass
		return(p,v,lst[1:])
	elif p==OP:
		if v in['+','-','!','~','#']:
			(p2,v2,lst2)=get_expr(lst[1:],defs,ban)
			if v=='#':
				if p2!=IDENT:raise PreprocError,"ident expected %s"%str(lst)
				return get_expr([(STR,v2)]+lst2,defs,ban)
			if p2!=NUM:raise PreprocError,"num expected %s"%str(lst)
			if v=='+':return(p2,v2,lst2)
			elif v=='-':return(p2,-int(v2),lst2)
			elif v=='!':return(p2,int(not int(v2)),lst2)
			elif v=='~':return(p2,~int(v2),lst2)
			return(p2,v2,lst2)
		elif v=='(':
			count_par=0
			i=0
			for _,v in lst:
				if v==')':
					count_par-=1
					if count_par==0:break
				elif v=='(':count_par+=1
				i+=1
			else:
				raise PreprocError,"rparen expected %s"%str(lst)
			ret=process_tokens(lst[1:i],defs,ban)
			if len(ret)==1:
				(p,v)=ret[0]
				return(p,v,lst[i+1:])
			else:
				raise PreprocError,"cannot reduce %s"%str(lst)
	elif p==IDENT:
		if len(lst)>1:
			(p2,v2)=lst[1]
			if v2=="##":
				(p3,v3)=lst[2]
				if p3!=IDENT and p3!=NUM and p3!=OP:
					raise PreprocError,"%s: ident expected after '##'"%str(lst)
				return get_expr([(p,v+v3)]+lst[3:],defs,ban)
		if v.lower()=='defined':
			(p2,v2)=lst[1]
			off=2
			if v2=='(':
				(p2,v2)=lst[2]
				if p2!=IDENT:raise PreprocError,'expected an identifier after a "defined("'
				(p3,v3)=lst[3]
				if v3!=')':raise PreprocError,'expected a ")" after a "defined(x"'
				off=4
			elif p2!=IDENT:
				raise PreprocError,'expected a "(" or an identifier after a defined'
			x=0
			if v2 in defs:x=1
			return(NUM,x,lst[off:])
		elif not v in defs or v in ban:
			if"waf_include"in ban:return(p,v,lst[1:])
			else:return(NUM,0,lst[1:])
		if type(defs[v])is types.StringType:
			v,k=extract_macro(defs[v])
			defs[v]=k
		macro_def=defs[v]
		if not macro_def[0]:
			lst=macro_def[1]+lst[3:]
			return get_expr(lst,defs,ban)
		else:
			params=[]
			i=1
			p2,v2=lst[i]
			if p2!=OP or v2!='(':raise PreprocError,"invalid function call '%s'"%v
			one_param=[]
			count_paren=0
			try:
				while 1:
					i+=1
					p2,v2=lst[i]
					if p2==OP and count_paren==0:
						if v2=='(':
							one_param.append((p2,v2))
							count_paren+=1
						elif v2==')':
							if one_param:params.append(one_param)
							lst=lst[i+1:]
							break
						elif v2==',':
							if not one_param:raise PreprocError,"empty param in funcall %s"%p
							params.append(one_param)
							one_param=[]
						else:
							one_param.append((p2,v2))
					else:
						one_param.append((p2,v2))
						if v2=='(':count_paren+=1
						elif v2==')':count_paren-=1
			except IndexError,e:
				raise
			accu=[]
			table=macro_def[0]
			for p2,v2 in macro_def[1]:
				if p2==IDENT and v2 in table:accu+=params[table[v2]]
				else:
					if v2=='__VA_ARGS__':
						va_toks=[]
						st=len(macro_def[0])
						pt=len(params)
						for x in params[pt-st+1:]:
							va_toks.extend(x)
							va_toks.append((OP,','))
						if va_toks:va_toks.pop()
						if len(accu)>1:
							(p3,v3)=accu[-1]
							(p4,v4)=accu[-2]
							if v3=='##':
								accu.pop()
								if v4==','and pt<st:
									accu.pop()
						accu+=va_toks
					else:
						accu.append((p2,v2))
			return get_expr(accu+lst,defs,ban+[v])
def process_tokens(lst,defs,ban):
	accu=[]
	while lst:
		p,v,nlst=get_expr(lst,defs,ban)
		if p==NUM:
			if not nlst:return[(p,v)]
			op1,ov1=nlst[0]
			if op1!=OP:
				raise PreprocError,"op expected %s"%str(lst)
			if ov1=='?':
				i=0
				count_par=0
				for _,k in nlst:
					if k==')':count_par-=1
					elif k=='(':count_par+=1
					elif k==':'and count_par==0:break
					i+=1
				else:raise PreprocError,"ending ':' expected %s"%str(lst)
				if reduce_nums(v,0,'+'):lst=nlst[1:i]
				else:lst=nlst[i+1:]
				continue
			elif ov1==',':
				lst=nlst[1:]
				continue
			p2,v2,nlst=get_expr(nlst[1:],defs,ban)
			if p2!=NUM:raise PreprocError,"num expected after op %s"%str(lst)
			if nlst:
				op3,ov3=nlst[0]
				if prec[ov3]<prec[ov1]:
					p4,v4,nlst2=get_expr(nlst[1:],defs,ban)
					v5=reduce_nums(v2,v4,ov3)
					lst=[(p,v),(op1,ov1),(NUM,v5)]+nlst2
					continue
			lst=[(NUM,reduce_nums(v,v2,ov1))]+nlst
			continue
		elif p==STR:
			if nlst:raise PreprocError,"sequence must terminate with a string %s"%str(nlst)
			return[(p,v)]
		return(None,None,[])
def eval_macro(lst,adefs):
	ret=process_tokens(lst,adefs,[])
	if not ret:raise PreprocError,"missing tokens to evaluate %s"%str(lst)
	p,v=ret[0]
	return int(v)!=0
class c_parser(object):
	def __init__(self,nodepaths=None,defines=None):
		self.lines=[]
		if defines is None:
			self.defs={}
		else:
			self.defs=dict(defines)
		self.state=[]
		self.env=None
		self.count_files=0
		self.deps=[]
		self.deps_paths=[]
		self.currentnode_stack=[]
		self.nodepaths=nodepaths or[]
		self.nodes=[]
		self.names=[]
		self.curfile=''
		self.ban_includes=[]
		try:
			self.parse_cache=Build.bld.parse_cache
		except AttributeError:
			Build.bld.parse_cache={}
			self.parse_cache=Build.bld.parse_cache
	def tryfind(self,filename):
		self.curfile=filename
		found=self.currentnode_stack[-1].find_resource(filename)
		for n in self.nodepaths:
			if found:
				break
			found=n.find_resource(filename)
		if not found:
			if not filename in self.names:
				self.names.append(filename)
			return
		self.nodes.append(found)
		if filename[-4:]!='.moc':
			self.addlines(found)
	def addlines(self,node):
		self.currentnode_stack.append(node.parent)
		filepath=node.abspath(self.env)
		self.count_files+=1
		if self.count_files>30000:raise PreprocError,"recursion limit exceeded, bailing out"
		pc=self.parse_cache
		debug('preproc: reading file %r'%filepath)
		try:
			lns=pc[filepath]
		except KeyError:
			pass
		else:
			self.lines=lns+self.lines
			return
		try:
			lines=filter_comments(filepath)
			lines.append((POPFILE,''))
			pc[filepath]=lines
			self.lines=lines+self.lines
		except IOError:
			raise PreprocError,"could not read the file %s"%filepath
		except Exception:
			if Logs.verbose>0:
				error("parsing %s failed"%filepath)
				traceback.print_exc()
	def start(self,node,env):
		debug('preproc: scanning %s (in %s)'%(node.name,node.parent.name))
		self.env=env
		variant=node.variant(env)
		self.addlines(node)
		if env['DEFLINES']:
			self.lines=[('define',x)for x in env['DEFLINES']]+self.lines
		while self.lines:
			(type,line)=self.lines.pop(0)
			if type==POPFILE:
				self.currentnode_stack.pop()
				continue
			try:
				self.process_line(type,line)
			except Exception,ex:
				if Logs.verbose:
					error("line parsing failed (%s): %s"%(str(ex),line))
					traceback.print_exc()
	def process_line(self,token,line):
		ve=Logs.verbose
		if ve:debug('preproc: line is %s - %s state is %s'%(token,line,self.state))
		state=self.state
		if token in['ifdef','ifndef','if']:
			state.append(undefined)
		elif token=='endif':
			state.pop()
		if not token in['else','elif','endif']:
			if skipped in self.state or ignored in self.state:
				return
		if token=='if':
			ret=eval_macro(tokenize(line),self.defs)
			if ret:state[-1]=accepted
			else:state[-1]=ignored
		elif token=='ifdef':
			m=re_mac.search(line)
			if m and m.group(0)in self.defs:state[-1]=accepted
			else:state[-1]=ignored
		elif token=='ifndef':
			m=re_mac.search(line)
			if m and m.group(0)in self.defs:state[-1]=ignored
			else:state[-1]=accepted
		elif token=='include'or token=='import':
			(type,inc)=extract_include(line,self.defs)
			if inc in self.ban_includes:return
			if token=='import':self.ban_includes.append(inc)
			if ve:debug('preproc: include found %s    (%s) '%(inc,type))
			if type=='"'or not strict_quotes:
				if not inc in self.deps:
					self.deps.append(inc)
				self.tryfind(inc)
		elif token=='elif':
			if state[-1]==accepted:
				state[-1]=skipped
			elif state[-1]==ignored:
				if eval_macro(tokenize(line),self.defs):
					state[-1]=accepted
		elif token=='else':
			if state[-1]==accepted:state[-1]=skipped
			elif state[-1]==ignored:state[-1]=accepted
		elif token=='define':
			m=re_mac.search(line)
			if m:
				name=m.group(0)
				if ve:debug('preproc: define %s   %s'%(name,line))
				self.defs[name]=line
			else:
				raise PreprocError,"invalid define line %s"%line
		elif token=='undef':
			m=re_mac.search(line)
			if m and m.group(0)in self.defs:
				self.defs.__delitem__(m.group(0))
		elif token=='pragma':
			if re_pragma_once.search(line.lower()):
				self.ban_includes.append(self.curfile)
def extract_macro(txt):
	t=tokenize(txt)
	if re_fun.search(txt):
		p,name=t[0]
		p,v=t[1]
		if p!=OP:raise PreprocError,"expected open parenthesis"
		i=1
		pindex=0
		params={}
		prev='('
		while 1:
			i+=1
			p,v=t[i]
			if prev=='(':
				if p==IDENT:
					params[v]=pindex
					pindex+=1
					prev=p
				elif p==OP and v==')':
					break
				else:
					raise PreprocError,"unexpected token"
			elif prev==IDENT:
				if p==OP and v==',':
					prev=v
				elif p==OP and v==')':
					break
				else:
					raise PreprocError,"comma or ... expected"
			elif prev==',':
				if p==IDENT:
					params[v]=pindex
					pindex+=1
					prev=p
				elif p==OP and v=='...':
					raise PreprocError,"not implemented"
				else:
					raise PreprocError,"comma or ... expected"
			elif prev=='...':
				raise PreprocError,"not implemented"
			else:
				raise PreprocError,"unexpected else"
		return(name,[params,t[i+1:]])
	else:
		(p,v)=t[0]
		return(v,[[],t[1:]])
re_include=re.compile('^\s*(<(?P<a>.*)>|"(?P<b>.*)")')
def extract_include(txt,defs):
	m=re_include.search(txt)
	if m:
		if m.group('a'):return'<',m.group('a')
		if m.group('b'):return'"',m.group('b')
	tokens=tokenize(txt)
	tokens=process_tokens(tokens,defs,['waf_include'])
	p,v=tokens[0]
	if p!=STR:raise PreprocError,"could not parse include %s"%txt
	return('"',v)
def parse_char(txt):
	if not txt:raise PreprocError,"attempted to parse a null char"
	if txt[0]!='\\':
		return ord(txt)
	c=txt[1]
	if c=='x':
		if len(txt)==4 and txt[3]in string.hexdigits:return int(txt[2:],16)
		return int(txt[2:],16)
	elif c.isdigit():
		if c=='0'and len(txt)==2:return 0
		for i in 3,2,1:
			if len(txt)>i and txt[1:1+i].isdigit():
				return(1+i,int(txt[1:1+i],8))
	else:
		try:return chr_esc[c]
		except KeyError:raise PreprocError,"could not parse char literal '%s'"%txt
def tokenize(s):
	ret=[]
	for match in re_clexer.finditer(s):
		m=match.group
		for name in tok_types:
			v=m(name)
			if v:
				if name==IDENT:
					try:v=g_optrans[v];name=OP
					except KeyError:
						if v.lower()=="true":
							v=1
							name=NUM
						elif v.lower()=="false":
							v=0
							name=NUM
				elif name==NUM:
					if m('oct'):v=int(v,8)
					elif m('hex'):v=int(m('hex'),16)
					elif m('n0'):v=m('n0')
					else:
						v=m('char')
						if v:v=parse_char(v)
						else:v=m('n2')or m('n4')
				elif name==OP:
					if v=='%:':v='#'
					elif v=='%:%:':v='##'
				ret.append((name,v))
				break
	return ret




# This file is part of the Hotwire Shell user interface.
#   
# Copyright (C) 2007 Colin Walters <walters@verbum.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, sys, re, logging, string

import gtk, gobject, pango

from hotssh.hotlib.logutil import log_except

def markup_for_match(text, start, end, matchtarget=None):
    source = matchtarget or text
    return  '%s<b>%s</b>%s%s' % (gobject.markup_escape_text(source[0:start]),
                                 gobject.markup_escape_text(source[start:end]),
                                 gobject.markup_escape_text(source[end:]),
                                 matchtarget and (' - <i>' + text + '</i>') or '')

_logger = logging.getLogger("hotwire.ui.QuickFind")

class QuickFindWindow(gtk.Dialog):
    def __init__(self, title, parent=None):
        super(QuickFindWindow, self).__init__(title=title,
                                              parent=parent,
                                              flags=gtk.DIALOG_DESTROY_WITH_PARENT,
                                              buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_ACCEPT))
        
        self.connect('response', lambda *args: self.hide())
        self.connect('delete-event', self.hide_on_delete)
                
        self.set_has_separator(False)
        self.set_border_width(5)
        
        self.__vbox = gtk.VBox()
        self.vbox.add(self.__vbox)   
        self.vbox.set_spacing(6)
        
        self.set_size_request(640, 480)
   
        self.__response_value = None
   
        self.__idle_search_id = 0
   
        self.__entry = gtk.Entry()
        self.__entry.connect('notify::text', self.__on_text_changed)
        self.__entry.connect('key-press-event', self.__on_keypress)
        self.__vbox.pack_start(self.__entry, expand=False)
        self.__scroll = gtk.ScrolledWindow()
        self.__scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.__results = gtk.TreeView()
        self.__results.connect('row-activated', self.__on_row_activated)
        self.__scroll.add(self.__results)
        colidx = self.__results.insert_column_with_data_func(-1, '',
                                                             gtk.CellRendererPixbuf(),
                                                             self.__render_icon)
        renderer = gtk.CellRendererText()
        renderer.set_property('ellipsize', True)
        colidx = self.__results.insert_column_with_data_func(-1, '',
                                                             renderer,
                                                             self.__render_match)
        frame = gtk.Frame()
        frame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        frame.add(self.__scroll)
        self.__vbox.pack_start(frame, expand=True)
        self.__selection = self.__results.get_selection()
        self.__selection.set_mode(gtk.SELECTION_SINGLE)
        self.__selection.connect('changed', self.__on_selection_changed)
        self.__results.set_headers_visible(False)
        
        self.__do_search()
        
    def __on_keypress(self, entry, event):
        if event.keyval == gtk.gdk.keyval_from_name('Return'):
            self.__idle_do_search()
            return self.__activate_selection()
        elif event.keyval == gtk.gdk.keyval_from_name('Up'):
            self.__idle_do_search()
            self.__select_next()
            return True
        elif event.keyval == gtk.gdk.keyval_from_name('Down'):
            self.__idle_do_search()
            self.__select_prev()
            return True
        return False        
        
    def __on_text_changed(self, *args):
        if self.__idle_search_id > 0:
            return
        self.__idle_search_id = gobject.timeout_add(300, self.__idle_do_search)
        
    def __idle_do_search(self):
        if self.__idle_search_id == 0:
            return False
        self.__idle_search_id = 0
        self.__do_search()
        return False
        
    def _markup_search(self, text, searchq, text_lower=None, searchq_lower=None):
        if text_lower is not None:
            idx = text_lower.find(searchq_lower)
        else:
            idx = text.find(searchq)
        if idx >= 0:
            return markup_for_match(text, idx, idx+len(searchq))
        else:
            return None          
        
    def _get_string_data(self):
        """Generate a list of strings for search data."""
        raise NotImplementedError()
    
    def _do_search(self, text):
        """Override this to implement a custom search with icons, etc."""
        strings = self._get_string_data()
        text = text.lower()
        for s in strings:
            if s.lower().find(text) >= 0:
                yield (s, s, None)
        
    def __do_search(self):
        text = self.__entry.get_property('text')
        results = self._do_search(text)
        model = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)
        have_results = False
        for result in results:
            have_results = True
            _logger.debug("appending result: %s", result)
            model.append(result)
        self.__results.set_model(model)
        if have_results:
            self.__selection.select_iter(model.get_iter_first())            
        
    def __on_selection_changed(self, sel):
        (model, iter) = sel.get_selected()
        if iter:
            self.__results.scroll_to_cell(model.get_path(iter))
            
    def __get_selected_path(self):
        (model, iter) = self.__selection.get_selected()
        return iter and model.get_path(iter)
        
    def __select_next(self):
        path = self.__get_selected_path()
        if not path:
            return
        previdx = path[-1]-1
        if previdx < 0:
            return
        model = self.__results.get_model()        
        previter = model.iter_nth_child(None, previdx)
        if not previter:
            return
        self.__selection.select_iter(previter)
        
    def __select_prev(self):
        path = self.__get_selected_path()
        if not path:
            return
        model = self.__results.get_model()        
        seliter = model.get_iter(path)
        iternext = model.iter_next(seliter)
        if not iternext:
            return
        self.__selection.select_iter(iternext)
        
    def __activate_selection(self):
        (model, iter) = self.__selection.get_selected()
        if not iter:
            return False
        self._handle_activation(model.get_value(iter, 0))
        return True        
        
    @log_except(_logger)
    def __on_row_activated(self, tv, path, vc):
        _logger.debug("row activated: %s", path)
        model = self.__results.get_model()
        iter = model.get_iter(path)
        self._handle_activation(model.get_value(iter, 0))
    
    def __render_match(self, col, cell, model, iter):
        markup = model.get_value(iter, 1)
        if markup:
            cell.set_property('markup', markup)
        else:
            cell.set_property('text', model.get_value(iter, 0))    
    
    def __render_icon(self, col, cell, model, iter):
        icon_name = model.get_value(iter, 2)
        if icon_name:
            pbcache = PixbufCache.getInstance()
            pixbuf = pbcache.get(icon_name, size=16, trystock=True, stocksize=gtk.ICON_SIZE_MENU)
            cell.set_property('pixbuf', pixbuf)
        else:
            cell.set_property('icon-name', None)    
    
    def get_response_value(self):
        return self.__response_value
        
    def _handle_activation(self, val):
        self.__response_value = val
        self.response(gtk.RESPONSE_ACCEPT)
    
    def run_get_value(self):
        self.show_all()
        resp = self.run()
        if resp == gtk.RESPONSE_ACCEPT:
            return self.__response_value
        return None



#! /usr/bin/env python
# encoding: utf-8

import os,optparse,sys,re
import Configure,Options,Utils
import ccroot,ar
from Configure import conftest
def find_gxx(conf):
	v=conf.env
	cxx=None
	if v['CXX']:cxx=v['CXX']
	elif'CXX'in os.environ:cxx=os.environ['CXX']
	if not cxx:cxx=conf.find_program('g++',var='CXX')
	if not cxx:cxx=conf.find_program('c++',var='CXX')
	if not cxx:conf.fatal('g++ was not found')
	try:
		if Utils.cmd_output('%s --version'%cxx).find('g++')<0:
			conf.fatal('g++ was not found, see the result of g++ --version')
	except ValueError:
		conf.fatal('g++ --version could not be executed')
	v['CXX']=cxx
	v['CXX_NAME']='gcc'
	ccroot.get_cc_version(conf,cxx,'CXX_VERSION')
def gxx_common_flags(conf):
	v=conf.env
	v['CXX_SRC_F']=''
	v['CXX_TGT_F']='-c -o '
	v['CPPPATH_ST']='-I%s'
	if not v['LINK_CXX']:v['LINK_CXX']=v['CXX']
	v['CXXLNK_SRC_F']=''
	v['CXXLNK_TGT_F']='-o '
	v['LIB_ST']='-l%s'
	v['LIBPATH_ST']='-L%s'
	v['STATICLIB_ST']='-l%s'
	v['STATICLIBPATH_ST']='-L%s'
	v['RPATH_ST']='-Wl,-rpath,%s'
	v['CXXDEFINES_ST']='-D%s'
	v['SHLIB_MARKER']='-Wl,-Bdynamic'
	v['STATICLIB_MARKER']='-Wl,-Bstatic'
	v['FULLSTATIC_MARKER']='-static'
	v['program_PATTERN']='%s'
	v['shlib_CXXFLAGS']=['-fPIC','-DPIC']
	v['shlib_LINKFLAGS']=['-shared']
	v['shlib_PATTERN']='lib%s.so'
	v['staticlib_LINKFLAGS']=['-Wl,-Bstatic']
	v['staticlib_PATTERN']='lib%s.a'
	v['MACBUNDLE_LINKFLAGS']=['-bundle','-undefined dynamic_lookup']
	v['MACBUNDLE_CCFLAGS']=['-fPIC']
	v['MACBUNDLE_PATTERN']='%s.bundle'
def gxx_modifier_win32(conf):
	if sys.platform!='win32':return
	v=conf.env
	v['program_PATTERN']='%s.exe'
	v['shlib_PATTERN']='lib%s.dll'
	v['shlib_CXXFLAGS']=['']
	v['staticlib_LINKFLAGS']=['']
def gxx_modifier_cygwin(conf):
	if sys.platform!='cygwin':return
	v=conf.env
	v['program_PATTERN']='%s.exe'
	v['shlib_PATTERN']='lib%s.dll'
	v['shlib_CXXFLAGS']=['']
	v['staticlib_LINKFLAGS']=['']
def gxx_modifier_darwin(conf):
	if sys.platform!='darwin':return
	v=conf.env
	v['shlib_CXXFLAGS']=['-fPIC','-compatibility_version 1','-current_version 1']
	v['shlib_LINKFLAGS']=['-dynamiclib']
	v['shlib_PATTERN']='lib%s.dylib'
	v['staticlib_LINKFLAGS']=['']
	v['SHLIB_MARKER']=''
	v['STATICLIB_MARKER']=''
def gxx_modifier_aix5(conf):
	if sys.platform!='aix5':return
	v=conf.env
	v['program_LINKFLAGS']=['-Wl,-brtl']
	v['shlib_LINKFLAGS']=['-shared','-Wl,-brtl,-bexpfull']
	v['SHLIB_MARKER']=''
detect='''
find_gxx
find_cpp
find_ar
gxx_common_flags
gxx_modifier_win32
gxx_modifier_cygwin
gxx_modifier_darwin
gxx_modifier_aix5
cxx_load_tools
cxx_add_flags
'''

conftest(find_gxx)
conftest(gxx_common_flags)
conftest(gxx_modifier_win32)
conftest(gxx_modifier_cygwin)
conftest(gxx_modifier_darwin)
conftest(gxx_modifier_aix5)
